/* ------------------------------------------------------------------------ */
/* DMONO.C (C) Copyright Bill Buckels 2012                                  */
/* All Rights Reserved.                                                     */
/*                                                                          */
/* Licence Agreement                                                        */
/* -----------------                                                        */
/*                                                                          */
/* You have a royalty-free right to use, modify, reproduce and              */
/* distribute this source code in any way you find useful,                  */
/* provided that you agree that Bill Buckels has no warranty obligations    */
/* or liability resulting from said distribution in any way whatsoever.     */
/* If you don't agree, remove this source code and related files            */
/* from your computer now.                                                  */
/*                                                                          */
/* Written by   : Bill Buckels                                              */
/* Email:         bbuckels@escape.ca                                        */
/*                                                                          */
/* Purpose      : "Round Trip" utility to convert between Windows 			*/
/*                 Monochrome BMP files and Apple II Mono DHGR files.		*/
/*                                                                          */
/* Purpose 1    : This utility will allow you to convert     				*/
/*                2 - color (mono) BMP files 560 x 192 or 560 x 384 to      */
/*                Apple II Double Hi-Res 560 x 192 monochrome images.		*/
/*																			*/
/* Purpose 2	: This utility will also allow you to convert			    */
/*				  Apple II Double Hi-Res 560 x 192 monochrome images in     */
/*                either .2FC format or BIN AUX format to                   */
/*                2 - color (mono) BMP files 560 x 192 or 560 x 384.        */
/*																			*/
/*                This utility also converts from AppleWin Mono             */
/*                screen capture 256 color BMPs to                          */
/*                2 - color (mono) BMP files 560 x 192 or 560 x 384.        */
/*																			*/
/*                During conversion of the AppleWin 256 color BMP it also   */
/*                generates Apple II Double Hi-Res 140 x 192 images         */
/*                and renames the AppleWin BMP.                             */
/*																			*/
/* Revision     : 1.0 First Release                                         */
/*                2.0 Added AppleWin Support                                */
/* ------------------------------------------------------------------------ */
/* Written in Large Model 16 bit Microsoft C (MSC) Version 8.00c            */
/* Note: Run in an MS-DOS emulator like DOSBox if you can't run it raw.     */
/* ------------------------------------------------------------------------ */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <fcntl.h>

/* Bitmap Header structures */
typedef struct tagRGBQUAD
{
    unsigned char    rgbBlue;
    unsigned char    rgbGreen;
    unsigned char    rgbRed;
    unsigned char    rgbReserved;
} RGBQUAD;

typedef struct tagBITMAPINFOHEADER
{
    unsigned long   biSize;
    unsigned long   biWidth;
    unsigned long   biHeight;
    unsigned int    biPlanes;
    unsigned int    biBitCount;
    unsigned long   biCompression;
    unsigned long   biSizeImage;
    unsigned long   biXPelsPerMeter;
    unsigned long   biYPelsPerMeter;
    unsigned long   biClrUsed;
    unsigned long   biClrImportant;
} BITMAPINFOHEADER;

/* constant for the biCompression field */
#define BI_RGB      0L

typedef struct tagBITMAPFILEHEADER
{
    unsigned char   bfType[2];
    unsigned long   bfSize;
    unsigned int    bfReserved1;
    unsigned int    bfReserved2;
    unsigned long   bfOffBits;
} BITMAPFILEHEADER;

/* static structures for processing AppleWin bmp input file */
BITMAPFILEHEADER BitMapFileHeader;
RGBQUAD          sbmp[256]; /* super vga - applewin */
BITMAPINFOHEADER bmi;



/* All of the Apple monitors are capable of supporting the maximum
   Apple II Double Hi-Res standard of 560x192 (mono).

   Back in the day...
   ibm-pc's sported a 640 x 200 CGA mono graphics display which was similar.

*/

/* The double hi-res display interleaves bytes from
the two different memory pages (auxiliary and  motherboard).
Seven bits from a byte in the auxiliary memory bank are  displayed first,
followed by seven bits from the corresponding byte on the  motherboard.
The bits are shifted out the same way as in standard hi-res  (least-significant bit first).
In double hi-res, the most significant bit of each byte is ignored.
*/

#define ASCIIZ 0


unsigned char *szTextTitle =
    "DMONO(C) Version 1.0 Copyright Bill Buckels 2012\n"
    "All Rights Reserved.";

#define SUCCESS  0
#define VALID    SUCCESS
#define FAILURE  -1
#define INVALID  FAILURE

/* provides base address for page1 hires scanlines  */
unsigned HB[]={
0x2000, 0x2400, 0x2800, 0x2C00, 0x3000, 0x3400, 0x3800, 0x3C00,
0x2080, 0x2480, 0x2880, 0x2C80, 0x3080, 0x3480, 0x3880, 0x3C80,
0x2100, 0x2500, 0x2900, 0x2D00, 0x3100, 0x3500, 0x3900, 0x3D00,
0x2180, 0x2580, 0x2980, 0x2D80, 0x3180, 0x3580, 0x3980, 0x3D80,
0x2200, 0x2600, 0x2A00, 0x2E00, 0x3200, 0x3600, 0x3A00, 0x3E00,
0x2280, 0x2680, 0x2A80, 0x2E80, 0x3280, 0x3680, 0x3A80, 0x3E80,
0x2300, 0x2700, 0x2B00, 0x2F00, 0x3300, 0x3700, 0x3B00, 0x3F00,
0x2380, 0x2780, 0x2B80, 0x2F80, 0x3380, 0x3780, 0x3B80, 0x3F80,
0x2028, 0x2428, 0x2828, 0x2C28, 0x3028, 0x3428, 0x3828, 0x3C28,
0x20A8, 0x24A8, 0x28A8, 0x2CA8, 0x30A8, 0x34A8, 0x38A8, 0x3CA8,
0x2128, 0x2528, 0x2928, 0x2D28, 0x3128, 0x3528, 0x3928, 0x3D28,
0x21A8, 0x25A8, 0x29A8, 0x2DA8, 0x31A8, 0x35A8, 0x39A8, 0x3DA8,
0x2228, 0x2628, 0x2A28, 0x2E28, 0x3228, 0x3628, 0x3A28, 0x3E28,
0x22A8, 0x26A8, 0x2AA8, 0x2EA8, 0x32A8, 0x36A8, 0x3AA8, 0x3EA8,
0x2328, 0x2728, 0x2B28, 0x2F28, 0x3328, 0x3728, 0x3B28, 0x3F28,
0x23A8, 0x27A8, 0x2BA8, 0x2FA8, 0x33A8, 0x37A8, 0x3BA8, 0x3FA8,
0x2050, 0x2450, 0x2850, 0x2C50, 0x3050, 0x3450, 0x3850, 0x3C50,
0x20D0, 0x24D0, 0x28D0, 0x2CD0, 0x30D0, 0x34D0, 0x38D0, 0x3CD0,
0x2150, 0x2550, 0x2950, 0x2D50, 0x3150, 0x3550, 0x3950, 0x3D50,
0x21D0, 0x25D0, 0x29D0, 0x2DD0, 0x31D0, 0x35D0, 0x39D0, 0x3DD0,
0x2250, 0x2650, 0x2A50, 0x2E50, 0x3250, 0x3650, 0x3A50, 0x3E50,
0x22D0, 0x26D0, 0x2AD0, 0x2ED0, 0x32D0, 0x36D0, 0x3AD0, 0x3ED0,
0x2350, 0x2750, 0x2B50, 0x2F50, 0x3350, 0x3750, 0x3B50, 0x3F50,
0x23D0, 0x27D0, 0x2BD0, 0x2FD0, 0x33D0, 0x37D0, 0x3BD0, 0x3FD0};

unsigned char dhrbuf[16384];

unsigned char msk[]={0x80,0x40,0x20,0x10,0x8,0x4,0x2,0x1};
unsigned char bmpscanline[72]; /* only the first 70 bytes are used */

unsigned char buf560[560];

/* decodes bmp scanline from file */
int ibmbits()
{
     int i,j,k;

     j=0;
     for(i=0;i<70;i++)
     {

        for(k=0;k<8;k++)
        {
            if(bmpscanline[i]&msk[k])buf560[j] = 1;
            else buf560[j] = 0;
            j++;
        }
     }
     return SUCCESS;

}

/* encodes apple II dhgr scanline into buffer */
int applebits(int y)
{
	    int xoff,idx,jdx;
	    unsigned char *ptraux, *ptrmain, bits[7];

	    xoff = HB[y];
	    ptraux  = (unsigned char *) &dhrbuf[xoff-0x2000];
	    ptrmain = (unsigned char *) &dhrbuf[xoff];

        xoff = 0;
        for (idx = 0; idx < 40; idx++) {

            for (jdx = 0; jdx < 7; jdx++) {
				bits[jdx] = buf560[xoff]; xoff++;
			}
			ptraux[idx] = (bits[6]<<6|bits[5]<<5|bits[4]<<4|
                           bits[3]<<3|bits[2]<<2|bits[1]<<1|bits[0]);

            for (jdx = 0; jdx < 7; jdx++) {
				bits[jdx] = buf560[xoff]; xoff++;
			}
			ptrmain[idx] = (bits[6]<<6|bits[5]<<5|bits[4]<<4|
                            bits[3]<<3|bits[2]<<2|bits[1]<<1|bits[0]);

		}
		return SUCCESS;

}


/* save both supported apple II dhgr files... */
/* save 2 apple II dhgr BIN and AUX output files */
/* and save 1 apple II dhgr 2FC output file */
int savedhrfragment(unsigned char *basename)
{

	FILE *fp;
	unsigned char outfile[128];

	sprintf(outfile,"%s.2FC",basename);
	fp = fopen(outfile,"wb");
	if (NULL == fp)return INVALID;
	fwrite(dhrbuf,1,16384,fp);
	fclose(fp);

    // the bsaved images are split into two files
    // the first file is loaded into aux mem
    sprintf(outfile,"%s.AUX",basename);
	fp = fopen(outfile,"wb");
	if (NULL == fp)return INVALID;
	fwrite(dhrbuf,1,8192,fp);
	fclose(fp);

    // the second file is loaded into main mem
    sprintf(outfile,"%s.BIN",basename);
	fp = fopen(outfile,"wb");
	if (NULL == fp)return INVALID;
	fwrite(&dhrbuf[8192],1,8192,fp);
	fclose(fp);

	return SUCCESS;
}

/* headers for monochrome BMP files */

/* 560 x 384 - the default output format */
unsigned char mono384[62] ={
0x42, 0x4D, 0x3E, 0x6C, 0x00, 0x00, 0x00, 0x00,
0x00, 0x00, 0x3E, 0x00, 0x00, 0x00, 0x28, 0x00,
0x00, 0x00, 0x30, 0x02, 0x00, 0x00, 0x80, 0x01,
0x00, 0x00, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00,
0x00, 0x00, 0x00, 0x6C, 0x00, 0x00, 0x00, 0x00,
0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
0x00, 0x00, 0xFF, 0xFF, 0xFF, 0x00};

/* 560 x 192 - verbatim output format */
unsigned char mono192[62] ={
0x42, 0x4D, 0x3E, 0x36, 0x00, 0x00, 0x00, 0x00,
0x00, 0x00, 0x3E, 0x00, 0x00, 0x00, 0x28, 0x00,
0x00, 0x00, 0x30, 0x02, 0x00, 0x00, 0xC0, 0x00,
0x00, 0x00, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00,
0x00, 0x00, 0x00, 0x36, 0x00, 0x00, 0x00, 0x00,
0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
0x00, 0x00, 0xFF, 0xFF, 0xFF, 0x00};


/* read bmp files in both supported resolutions */
int BMPMono_Read(unsigned char *basename)
{

	unsigned int temp, temp2;
	unsigned char bmpfile[128];
	int x, y, skip = 1;
	FILE *fp;

    sprintf(bmpfile,"%s.BMP",basename);
	fp=fopen(bmpfile,"rb");

	if (NULL == fp) return INVALID;

    /* check the bmp header */
	for (x=0; x < 46; x++) {
	  temp2 = fgetc(fp);
	  temp=mono384[x];
	  if (temp != temp2) {
		rewind(fp);
		skip = 0;
		break;

	  }
	}
	if (skip == 0) {
		for (x=0; x < 46; x++) {
		  temp2 = fgetc(fp);
		  temp=mono192[x];
		  if (temp != temp2) {
			fclose(fp);
			return INVALID;

		  }
		}
	}

    /* throw away the next 16 bytes */
    /* we don't need them */
	/* ignore variant fields in header - 8 bytes */
	/* bmiHeader.biClrUsed and bmiHeader.biClrImportant */
	/* ignore 2 color palette - it's black and white - 8 bytes */
	/* RGB Quad Colors in palette = 4 bytes each. */
	for (x=0; x < 16; x++)fgetc(fp);

    /* remap the image to apple2 double hires mono format*/
	for (y = 192; y > 0; y--) {
	  	  for (x = 0; x < 72; x++) {
			  bmpscanline[x] = fgetc(fp);
		  }
		  /* skip every second scanline if 384 scanlines */
		  if (skip == 1) {
			  for (x = 0; x < 72; x++) {
				  bmpscanline[x] = fgetc(fp);
			  }
		  }
		  ibmbits(); // decode 8 bit
		  applebits(y-1); // encode 7 bit
	}

	fclose(fp);

  	return SUCCESS;
}

/* read 2 apple II dhgr input files */
int read_binaux(unsigned char *basename)
{

	FILE *fp;
	unsigned char outfile[128];

    // the bsaved images are split into two files
    // the first file is loaded into aux mem
    sprintf(outfile,"%s.AUX",basename);
	fp = fopen(outfile,"rb");
	if (NULL == fp)return INVALID;
	fread(dhrbuf,1,8192,fp);
	fclose(fp);

    // the second file is loaded into main mem
    sprintf(outfile,"%s.BIN",basename);
	fp = fopen(outfile,"rb");
	if (NULL == fp)return INVALID;
	fread(&dhrbuf[8192],1,8192,fp);
	fclose(fp);

	return SUCCESS;

}

/* read one apple II dhgr input file */
int read_2fc(unsigned char *basename)
{
	FILE *fp;
	unsigned char outfile[128];

    sprintf(outfile,"%s.2FC",basename);
	fp = fopen(outfile,"rb");
	if (NULL == fp)return INVALID;
	fread(dhrbuf,1,16384,fp);
	fclose(fp);

	return SUCCESS;

}

/* decodes apple II dhgr buffer */
int applebites(int y)
{
	    int xoff,idx;
	    unsigned char *ptraux, *ptrmain, ch;

	    xoff = HB[y];
	    ptraux  = (unsigned char *) &dhrbuf[xoff-0x2000];
	    ptrmain = (unsigned char *) &dhrbuf[xoff];

        xoff = 0;
        for (idx = 0; idx < 40; idx++) {

            ch = ptraux[idx];

            buf560[xoff] = ((ch) &1); xoff++;
            buf560[xoff] = ((ch >> 1) &1); xoff++;
            buf560[xoff] = ((ch >> 2) &1); xoff++;
            buf560[xoff] = ((ch >> 3) &1); xoff++;
            buf560[xoff] = ((ch >> 4) &1); xoff++;
            buf560[xoff] = ((ch >> 5) &1); xoff++;
            buf560[xoff] = ((ch >> 6) &1); xoff++;

            ch = ptrmain[idx];

            buf560[xoff] = ((ch) &1); xoff++;
            buf560[xoff] = ((ch >> 1) &1); xoff++;
            buf560[xoff] = ((ch >> 2) &1); xoff++;
            buf560[xoff] = ((ch >> 3) &1); xoff++;
            buf560[xoff] = ((ch >> 4) &1); xoff++;
            buf560[xoff] = ((ch >> 5) &1); xoff++;
            buf560[xoff] = ((ch >> 6) &1); xoff++;

 		}
		return SUCCESS;

}

/* encodes bmp scanline */
int ibmbites()
{
     int i,j,k;
     unsigned char bits[8], ch;

     j=0;
     for(i=0;i<70;i++)
     {
        ch = 0;
        for(k=0;k<8;k++)
        {
		  bits[k] = buf560[j]; j++;
		}
		bmpscanline[i] = (bits[0]<<7|bits[1]<<6|bits[2]<<5|bits[3]<<4|
                          bits[4]<<3|bits[5]<<2|bits[6]<<1|bits[7]);
     }
     return SUCCESS;

}

/* save a monochrome bmp file in either supported resolution */
int save_to_bmp(unsigned char *basename, int bmp384)
{

    FILE *fp;
    unsigned char outfile[128];
	int y,y2;


    sprintf(outfile,"%s.BMP",basename);
	fp = fopen(outfile,"wb");
	if (NULL == fp)return INVALID;

    /* write header */

    if (bmp384 == 1)
    	fwrite(mono384,1,sizeof(mono384),fp);
    else
        fwrite(mono192,1,sizeof(mono192),fp);

    /* write scanlines */

    y2 = 191;
    bmpscanline[70] = bmpscanline[71] = 0xff; // padding
   	for (y = 0; y< 192; y++) {
       applebites(y2);
       ibmbites();
       fwrite(bmpscanline,1,72,fp);
       /* double-up */
	   if (bmp384 == 1)
    		fwrite(bmpscanline,1,72,fp);
	   y2 -= 1;
    }

    return SUCCESS;

}


int ReadAppleWin(unsigned char *fullname, unsigned char *basename)
{

 	FILE *fp;
    int packet = INVALID, y,i,j;
    char bmpfile[128], outfile[128];

    /* if file does not exist just exit and save the
       other routines the trouble... */
    if((fp=fopen(fullname,"rb"))==NULL) {
		    puts(szTextTitle);
		    printf("%s cannot be opened.\n", fullname);
		    exit(1);
	}

    /* if not a bmp file then let the other routines take over */
    sprintf(bmpfile,"%s.BMP",basename);
    if((fp=fopen(bmpfile,"rb"))==NULL) {
		return INVALID;
	}

    /* read the header stuff into the appropriate structures,
       it's likely a bmp file */
    memset(&BitMapFileHeader.bfType,0,sizeof(BITMAPFILEHEADER));
    memset(&bmi.biSize,0,sizeof(BITMAPINFOHEADER));

    fread((char *)&BitMapFileHeader.bfType,
	             sizeof(BITMAPFILEHEADER),1,fp);
    fread((char *)&bmi.biSize,
                 sizeof(BITMAPINFOHEADER),1,fp);

    /* check for an applewin bmp */
    if (bmi.biCompression==BI_RGB &&
        BitMapFileHeader.bfType[0] == 'B' && BitMapFileHeader.bfType[1] == 'M' &&
        bmi.biPlanes==1 && bmi.biBitCount == 8 &&
        bmi.biWidth == 560 &&  bmi.biHeight == 384)  {
		fread((char *)&sbmp[0].rgbBlue, sizeof(RGBQUAD)*256,1,fp);
		if (sbmp[17].rgbRed == 255 && sbmp[17].rgbGreen == 255 && sbmp[17].rgbBlue == 254) packet = 560;
	}

    if(packet == INVALID){
		puts(szTextTitle);
		printf("%s is not a Supported 560 x 384 AppleWin Screen Capture.\n", bmpfile);
		fclose(fp);
		exit(1);
	}

	memset(&dhrbuf[0],0,16384); /* clear write buffer */

    i = 191; /* bmp's are upside-down
                so conversion of scanlines is in
                reverse order */
    for(y=192;y>0;y--)
    {
	  fread((char *)&buf560[0],1,560,fp);
	  /* skip every second scanline  */
	  fread((char *)&buf560[0],1,560,fp);
	  for (j=0;j<560;j++) {
	  	if (buf560[j] != 0)buf560[j] = 1;
	  }
	  applebits(i); // encode 7 bit mono line
      i--;
      if (i< 0) break;
    }
    fclose(fp);

    sprintf(outfile,"%s.2FC",basename);
	fp = fopen(outfile,"wb");
	if (NULL == fp) {
		puts(szTextTitle);
		printf("%s cannot be created.\n", outfile);
		exit(1);
	}
	fwrite(dhrbuf,1,16384,fp);
	fclose(fp);

    // the bsaved images are split into two files
    // the first file is loaded into aux mem
    sprintf(outfile,"%s.AUX",basename);
	fp = fopen(outfile,"wb");
	if (NULL == fp) {
		puts(szTextTitle);
		printf("%s cannot be created.\n", outfile);
		exit(1);
	}
	fwrite(dhrbuf,1,8192,fp);
	fclose(fp);

    // the second file is loaded into main mem
    sprintf(outfile,"%s.BIN",basename);
	fp = fopen(outfile,"wb");
	if (NULL == fp) {
		puts(szTextTitle);
		printf("%s cannot be created.\n", outfile);
		exit(1);
	}
	fwrite(&dhrbuf[8192],1,8192,fp);
	fclose(fp);

	sprintf(outfile,"%s.BM2",basename);
	remove(outfile);
	rename(bmpfile,outfile);

    sprintf(outfile,"%s.BIN",basename);
    strcpy(fullname,outfile);

return SUCCESS;

}

void main(int argc, char **argv)
{
  int status = 0, idx, bmp = 0, a2fc = 0, auxbin = 0, bmp384 = 1;

  unsigned char fname[128],sname[128],outfile[128], argbuf[128], c, d, e;
  unsigned char *wordptr;
  unsigned char scratchbuf[128];
  FILE *fp;

  system("cls");

  if(argc == 1) {
    puts(szTextTitle);
    puts("Command line Usage is \"DMONO MyHires.2FC\"");
    puts("                      \"DMONO MyHires.BIN\"");
    puts("                      \"DMONO MyHires.AUX\"");
	puts("                      \"DMONO MyAppleWinDHiresMonoScreenCapture.\"");
    puts("Default output BMP is 560 x 384");
    puts("Optional argument for 560 x 192 (squashed) output:");
    puts("                      \"DMONO MyInputFile 192\"");
    puts("To output Mono DHGR files from a 560 x 384 or 560 x 192 BMP:");
    puts("                      \"DMONO MyMono.BMP\"");
    puts("If converting .BIN and .AUX file pairs, both must be present.");
    puts("All input filenames must be entered with extensions.");
    printf("Enter Input FileName (Blank to Exit): ");
    gets(fname);
    if (fname[0] == ASCIIZ)
      exit(1);
  }
  else {
    strcpy(fname, argv[1]);
    if (argc > 2) {
	    strcpy(argbuf, argv[2]);
	    if (argbuf[0] == '1' && argbuf[1] == '9' && argbuf[2] == '2')bmp384 = 0;
	}
  }

  for (idx = 0; fname[idx] != ASCIIZ; idx++) {

      if (fname[idx] == '.') {
		  c = toupper(fname[idx + 1]);
		  d = toupper(fname[idx + 2]);
		  e = toupper(fname[idx + 3]);
		  /* support for our extensions only */
		  if (c == 'B' && d == 'M' && e == 'P') bmp = 1;
		  if (c == '2' && d == 'F' && e == 'C') a2fc = 1;
		  if (c == 'B' && d == 'I' && e == 'N') auxbin = 1;
		  if (c == 'A' && d == 'U' && e == 'X') auxbin = 1;
		  break;
	  }

  }

  strcpy(sname, fname);
  wordptr = strtok(sname, ".");
  strcpy(outfile,sname);


  switch(bmp)
  {
	  case 1:   status = ReadAppleWin(fname,sname);

                if (status) {

					status = BMPMono_Read(sname);

					if (status) {
					  puts(szTextTitle);
					  printf("%s is an Unsupported Format or cannot be opened.\n", fname);
					  exit(1);
					}
					status = savedhrfragment(outfile);
					if (status == SUCCESS)printf("%s.AUX & .BIN and %s.2FC Saved!\n",outfile,outfile);
					else printf("Error saving %s.AUX & .BIN and %s.2FC!\n",outfile,outfile);

					break;
				}

	  default: if (auxbin == 0 && a2fc == 0 && bmp == 0) {
				  puts(szTextTitle);
				  printf("%s is an Unsupported Format. File extension not recognized.\n", fname);
				  exit(1);
				}
				if (auxbin == 1) status = read_binaux(sname);
				if (a2fc == 1) status = read_2fc(sname);
				if (status) {
				  puts(szTextTitle);
				  printf("%s is an Unsupported Format or cannot be opened.\n", fname);
				  exit(1);
				}

	            status = save_to_bmp(outfile, bmp384);
	   			if (status == SUCCESS)printf("%s.BMP Saved!\n",outfile);
	   			else printf("Error saving %s.BMP!\n",outfile);
	            break;

  }

  puts("Have a Nice Dos!");
  exit(0);

}
