
/* ------------------------------------------------------------------------
System       : Manx Aztec C65 Version 3.2b
               MS-DOS cross-development environment
Platform     : Apple IIe 128K PRODOS 8
Program      : cubes.c
Description  : G2 Library Demo Program
               Isometric Cube Drawing Program in
               Double Hi-Res 560 x 192 monochrome mode.
Written by   : Bill Buckels
Date Written : January 2013
Revision     : 1.0 First Release
Licence      : You may use this routine for whatever you wish as long
               as you agree that Bill Buckels has no warranty or
               liability obligations whatsoever from said use.
------------------------------------------------------------------------ */


/* This program calculates the dimensions required to produce a
"Worms's Eye View" of an isometric cube from height, length, and
angle and then draws the cube.

It is based on a test program called CUBES.BAS that I wrote in BASIC
for the IBM-PC in October, 1989 when I was first developing
commercial 3-D simulations for "truecolor" Kiosk Systems for the Home
Decorating Industry. Today it makes a reasonable demo.

In passing, I should also note that CUBES.BAS used cartesian
coordinates rather than screen coordinates which resulted in a bird's
eye view rather than a worm's eye view of the isometric cube.

Also, for this demo I have renamed the variables from CUBES.BAS where
they might be confusing and adjusted the limits appropriately for the
apple II display using a 480 x 480 viewport for my calculations and
then I scale the yterm by 1:3 for plotting which results in a 480 x
160 display resolution for the cube. CUBES.BAS was targeted at a 512
x 512 display and used a scaled viewport which did not require
scaling adjustments for the display since scaling was done by the
BASIC graphics library.

CUBES.BAS also filled the facets of the cube with fill patterns.
Due to laziness on my part I have never written a fill function
for any of the Apple II screen modes in my Aztec C graphics library
so for this demo I have simply ported the line drawing calculations
and line plotting and make no attempt to fill the cube.

*/

double atof();
double height, length, angle;

cube(color)
int color;
{

  double baselength, elev, xorg, yorg, midbot, midtop, botside, boxbot, ltedge, rtedge, htedge;
  int x, x1, x2, y, y1, y2, y3;

  /* reduce the length by the rate of deflection in degrees from base 0 */
  baselength=(length-((length/90)*angle));

  /* subtract the rate of deflection from the height */
  elev=((height/90)*angle);

  /* offset the cube from the middle of the screen */
  xorg=(240.0);             /* horizontal viewpoint */
  yorg=(240.0-elev);        /* vertical viewpoint */
  midtop= (yorg-(height/2));/* top apex */
  botside=(height+midtop);  /* leading edge */
  midbot= (botside+elev);   /* bottom dead centre */
  boxbot= (midbot+elev);    /* bottom apex */
  ltedge= (xorg-baselength);/* left hand length truncation */
  rtedge= (xorg+baselength);/* right hand length truncation */
  htedge= (elev+midtop);    /* elevation at edges */

  /* draw the cube left to right, bottom to top */

  /* centre xterm on view from 240 to 280 */
  x = (int)(40.0 + xorg);
  x1 = (int)(40.0 + rtedge);
  x2 = (int)(40.0 + ltedge);

  /* scale down yterm from 480 to 160 */
  y = (int) (midbot /3);
  y1 = (int) (boxbot / 3);
  y2 = (int) (botside / 3);

  /* draw the bottom */
  dhmonoline(x2,y,x,y1,color);
  dhmonoline(x,y1,x1,y,color);
  dhmonoline(x1,y,x,y2,color);
  dhmonoline(x,y2,x2,y,color);

  /* scale down yterm from 480 to 160 */
  y1 = (int) (midtop / 3);
  y3 = (int) (htedge /3);

  /* draw the left side */
  dhmonoline(x2,y,x2,y3,color);
  dhmonoline(x2,y3,x,y1,color);
  dhmonoline(x,y1,x,y2,color);
  dhmonoline(x,y2,x2,y,color);

  /* draw the right side */
  dhmonoline(x1,y,x1,y3,color);
  dhmonoline(x1,y3,x,y1,color);
  dhmonoline(x,y1,x,y2,color);
  dhmonoline(x,y2,x1,y,color);

  return;

}

main()
{


 char buf[30];
 int drawcolor=1, floodcolor=0, h, l, a;


 /* set double hires */
 dhireson();
 dhrflood(0,0,139,191,floodcolor);
 dloclear_bottom();
 dlomixed();

 for (;;) {
	dmonoplots("Cubes - a Monochrome DHGR demo in Aztec C by Bill Buckels.",279,0,drawcolor,'M');
 	/* set cursor to bottom row */
 	scr_curs (23, 0);

    /* viewport set to 480 x 480 */
    printf("Enter blanks for default...\n");
 	printf("Enter box Height (10 to 240): "); gets(buf); height=atof(buf);
 	printf("Enter box Length (10 to 240): "); gets(buf); length=atof(buf);
 	printf("Enter box Angle  (10 to 80):  ");  gets(buf); angle=atof(buf);

    a = (int)angle;
    l = (int)length;
    h = (int)height;

	if (a < 10 || a > 80) angle = 30.0;
    if (l < 10 || l > 240) length = 240.0;
	if (h < 10 || h > 240) height = 240.0;

    a = (int)angle;
    l = (int)length;
    h = (int)height;

    dloclear_bottom();
    scr_curs (23, 0);
    printf("Height %d, Length %d, Angle %d... Please Wait.",h,l,a);

    cube(drawcolor);

  	printf("\nPress Esc to exit, any other key to continue...");
  	if (getch()==27)break;

 	if (drawcolor == 1) {
	  drawcolor = 0;
	  floodcolor = 15;
	}
	else {
	  drawcolor = 1;
	  floodcolor = 0;
	}

  	dhrflood(0,0,139,159,floodcolor);
  	dloclear_bottom();

	}

 /* return to the shell */
 dhrflood(0,0,139,191,0);
 dloresoff();
 scr_clear();
 scr_curs (0, 0);
 exit(0);

}