/* ------------------------------------------------------------------------
System       : Manx Aztec C65 Version 3.2b
               MS-DOS cross-development environment
Platform     : Apple IIe 128K PRODOS 8
Program      : dlode(C) Copyright Bill Buckels 2013.
               All rights reserved.
Description  : Apple //e ProDOS 8 DHGR Command Line Picture Viewer
               for the Aztec C ProDOS Shell.

               dlode commandline usage: dlode -seconds DHGRimagename.2FC
			                            dlode -seconds DHGRimagename.BIN
			   dlode interactive usage: dlode

			   DHGR formats supported:  A2FC or BIN AUX pairs.

Written by   : Bill Buckels
Date Written : January 2013
Revision     : 1.0 First Release
Licence      : You may use this program for whatever you wish as long
               as you agree that Bill Buckels has no warranty or
               liability obligations whatsoever from said use.
------------------------------------------------------------------------ */

#include <g3.h>
#include <console.h>

char *usage =
"dlode(C) Copyright Bill Buckels 2013.All rights reserved.\n"
"Command Line: dlode DHGRimagename or dlode -seconds DHGRimagename\n"
"  Multiple images can be entered for slideshow\n"
"  Slideshow Timeout is in seconds (default = 5)\n"
"DHGR formats supported: A2FC or BIN AUX pairs.\n";


main (argc, argv)
int argc;
char *argv[];
{

	char picname[64], *ptr;
	int c= -3, cnt, idx, numpics = 1, timeout = 0, skip = 0, key;

	scr_clear();
	scr_curs (0, 0);

	if (argc == 1) {
		mode80();
		printf(usage);
		printf("Enter DHGR Pic Name: ");
		gets(picname); /* get the filename */
		/* blank line to end program */
    	scr_clear();
		scr_curs (0, 0);
		if (picname[0] == 0)exit(c);
	}
	else {
		if (argc > 2) {
			/* slideshow timeout 1-100 if more than one image loaded from the
			   commandline - default is 5 seconds (approx.) on a "stock" apple II
			   and if the user has an accelerator or wishes something different
			   this can be adjusted as necessary. */
			timeout = 5;

			for (cnt = 1; cnt < argc; cnt++) {
				/* switch character is optional */
				if (argv[cnt][0] == '-')ptr = (char *)&argv[cnt][1];
				else ptr = (char *)&argv[cnt][0];
				idx = atoi(ptr);
				if (idx > 0) {
					skip = cnt;
					if (idx > 100) idx = 100;
					timeout = idx;
					break;
				}
			}
		}
	}



	/* set double hires */
	dhireson();
	/* clear screen */
	dxdblock(0,191,0);

	cnt = 1;
    while (kbhit())clearkey();

	for (;;) {


        /* preload keycheck */
        key = kbhit();
		if (key!=0) {
			clearkey();
			if (key == ESCAPE) break; /* if escape key is pressed end slideshow */
			key = 0;
		}

		if (argc > 1) strcpy(picname,argv[cnt]);

	    if (skip != cnt) {
			/* load picture */
			c = dlode(picname);

			/* if single picture loaded
			    1. if a problem just exit,
			    2. otherwise wait for key press and exit */
	   		if (argc < 3) {
				if (c!= 0) break;
	            getch();
	            break;
			}

            /* advance on timeout or advance, reverse or end on keypress */
	   	    if (c==0) key = wait(timeout,XTIME,YTIME);

	   	    /* special case - single image from command line with timeout */
	   	    if ((argc-skip) < 3) break;

		}
		else {
			key = kbhit();
			if (key!=0)clearkey();

		}

		/* postload key check */
		if (key == ESCAPE) break; /* if escape key is pressed end slideshow */

	   	switch(key) {
         	case UPARROW : /* reverse direction */
            case LTARROW : cnt--;
                           if (cnt < 1) cnt = (argc - 1);

                           /* the skip test - check twice
                              but even I can't think of everything */
                           if (cnt == skip) cnt--;
                           if (cnt < 1) cnt = (argc - 1);

                           if (cnt == skip) cnt--;
                           if (cnt < 1) cnt = (argc - 1);
                           break;
			default: cnt++;
		}

        if (cnt < argc) continue;
        cnt = 1;
	}

	while (kbhit())clearkey();


	dxdblock(0,191,0);

    dloresoff();
    scr_clear();
    scr_curs (0, 0);

    /* if the last picture did not load let them know
	   before exiting */
	if (c!=0) {
	  bellerr();
	  if (c == -1) printf("! Can't open %s!\n",picname);
	  else printf("! %s wrong format! Return code %d.\n",picname, c);
	  printf("Press Any Key...");
	  getch();
 	  scr_clear();
      scr_curs (0, 0);

	 }

	exit(c);

}
