/* ------------------------------------------------------------------------ */
/* MINI2BMP.C (C) Copyright Bill Buckels 2013                               */
/* All Rights Reserved.                                                     */
/*                                                                          */
/* Licence Agreement                                                        */
/* -----------------                                                        */
/*                                                                          */
/* You have a royalty-free right to use, modify, reproduce and              */
/* distribute this source code in any way you find useful,                  */
/* provided that you agree that Bill Buckels has no warranty obligations    */
/* or liability resulting from said distribution in any way whatsoever.     */
/* If you don't agree, remove this source code from your computer now.      */
/*                                                                          */
/* Written by   : Bill Buckels                                              */
/* Email:         bbuckels@escape.ca                                        */
/*                                                                          */
/* Purpose      : Legacy Graphics Converter                                 */
/*																			*/
/*                This program converts Apple IIe format Beagle Bros.and    */
/*				  "Old PrintShop" and "New Printshop" Minipix to the		*/
/*				  equivalent 16 color Windows BMP file suitable for			*/
/*				  coloring in windows paint and converting to Apple II		*/
/*				  native mode DLGR screen image fragments using the			*/
/*				  AppleX Aztec C65 distribution's BMP2LO.exe utility.		*/
/*                                                                          */
/*                I assume of course that you have some way of getting      */
/*                these files from an Apple II.                             */
/*																			*/
/* Revision     : 1.0 First Release                                         */
/* ------------------------------------------------------------------------ */
/* Written in Large Model 16 bit Microsoft C (MSC) Version 8.00c            */
/* Note: Run in an MS-DOS emulator like DOSBox if you can't run it raw.     */
/* ------------------------------------------------------------------------ */

#include <stdio.h>
#include <stdlib.h>
#include <fcntl.h>
#include <string.h>
#include <io.h>

#define SUCCESS 0
#define INVALID -1

#define ASCIIZ '\x00'

/*

The Apple ][ Minipix Format

When The Print Shop first appeared on The Apple ][, the only type of
graphics it used were small 4 sector DOS3.3 files, called minipix.
Because the range of them built into The Print Shop was limited, people
drew their own, using the print Shop Graphic Editor, and soon these
little 88 x 52 pictures were everywhere. There were disks and disks, just
packed with them. The storage format was quite simple, basically just a
bit mapped image, with every 11 bytes of the file mapping to one line of
88 dots on the screen. An extra 4 filler bytes were tacked on the end of
each mini-graphic file. Each file held only 1 mini-graphic. 52 lines x 8
bytes + 4 = 576 bytes. Now when "The New Print Shop" came along, you
could convert these little graphics to the new format. It turns out that
all that was done to each graphic to convert it, was a small header was
placed at the start, and the 4 filler bytes were dropped from the end,
and so 576 byte graphics became 605 byte "NPS" compatible graphics.

*/


/* As noted above, 2 Minipix formats exist. This program will convert
   either. This code in part is ripped-out of the mini2rag Aztec C65
   program which in part was ripped-out of the ClipShop Windows program
   which in part was ripped out of the many other printshop graphics
   converters that I have written since the late 1980's.
*/



/* I just arbitrarily read these based on filesize...
   if they convert that's fine...
   If they don't no harm done... GIGO */

#define NEW_CHUNK 605
#define OLD_CHUNK 576
#define NEW_HEADER 33
#define OLD_HEADER 0

unsigned char MiniPic[NEW_CHUNK+1];
int minihead = OLD_HEADER;

/* read an apple II minipic */
int MiniRead(char *fullname)
{
   int fh, c;
   FILE *fp;

   fp = fopen(fullname,"rb");
   if (NULL == fp) return INVALID;
   fh = fileno(fp);
   c = read(fh,MiniPic,NEW_CHUNK+1);
   fclose(fp);

   switch(c) {
	   case NEW_CHUNK:
	      	minihead = NEW_HEADER;
	   case OLD_CHUNK:
	      	c = SUCCESS;
	      break;
	   default:
	      c = -2;
	      break;
   }
   return (c);

}

unsigned char bmp_16[118] ={
 66,  77, 102,   9,   0,   0,   0,   0,
  0,   0, 118,   0,   0,   0,  40,   0,
  0,   0,  88,   0,   0,   0,  52,   0,
  0,   0,   1,   0,   4,   0,   0,   0,
  0,   0, 240,   8,   0,   0,   0,   0,
  0,   0,   0,   0,   0,   0,   0,   0,
  0,   0,   0,   0,   0,   0,   0,   0,
  0,   0,   0,   0, 128,   0,   0, 128,
  0,   0,   0, 128, 128,   0, 128,   0,
  0,   0, 128,   0, 128,   0, 128, 128,
  0,   0, 128, 128, 128,   0, 192, 192,
192,   0,   0,   0, 255,   0,   0, 255,
  0,   0,   0, 255, 255,   0, 255,   0,
  0,   0, 255,   0, 255,   0, 255, 255,
  0,   0, 255, 255, 255,   0};

unsigned char bmpbuf[4576];


int mini2bmp(unsigned char *basename)
{
	FILE *fp;
	int x,y,y1,byteoff,idx;
    unsigned char msk[]={0x80,0x40,0x20,0x10,0x8,0x4,0x2,0x1};
    unsigned char c, temp, outfile[128];

	sprintf(outfile,"%s.BMP",basename);
	fp = fopen(outfile,"wb");
	if (NULL == fp)return INVALID;

    for (x = 0; x < 118; x++) fputc(bmp_16[x],fp);
    memset(bmpbuf,0xff,4576);

	/* yterm */
	y1 = 51;
	for (y = 0, y1 = 51; y < 52; y++,y1--) {
		/* xterm */
	    x = 0;
	    byteoff = 88 * y1;

	    for (x = 0; x < 11; x++) {
	        /* get the next byte in the MiniPic */
	        c = MiniPic[minihead];
			minihead++;
            /* set each pixel in the BMP buffer
               by translating from 8 bit monochrome graphics */
			for (idx = 0; idx < 8; idx++) {
				if (c&msk[idx]) bmpbuf[byteoff] = ASCIIZ;
				byteoff++;
			}
		}
	}

    byteoff = 0;
    for (y = 0; y < 52; y++) {
    	for (x = 0; x < 88; x++) {
			if (x%2 == 0) {
			   temp = (bmpbuf[byteoff] << 4);
			}
			else {
			   c = (temp | bmpbuf[byteoff]);
			   fputc(c, fp);
			}
			byteoff++;
		}
	}

	fclose(fp);

	return SUCCESS;
}

void main(int argc, char **argv)
{
	int status = 0;
	unsigned char fullname[128],shortname[128],outfile[128];
	unsigned char *wordptr;

	system("cls");
	puts("MINI2BMP.C Version 1.0 (C) Copyright Bill Buckels 2013.\nAll Rights Reserved.");

	if(argc == 1) {
		puts("Usage is \"MINI2BMP MyAppleIIMiniPic [optional - outfile basename]\"\n"
		 	 "MS-DOS file names only (8 character basenames) - No long names!");
		printf("Enter Minipic FileName (Blank to Exit): ");
		gets(fullname);
		if (fullname[0] == ASCIIZ)
	  	exit(1);
		printf("Enter Output BMP BaseName (Blank for Same) : ");
		gets(outfile);
	}
	else {
		outfile[0] = ASCIIZ;
		strcpy(fullname, argv[1]);
		if (argc > 2) strcpy(outfile, argv[2]);
	}

	strcpy(shortname, fullname);
	wordptr = strtok(shortname, ".");
	if (outfile[0] == ASCIIZ)strcpy(outfile,shortname);

  	status = MiniRead(fullname);
	if (status!=0) {
		if (status == INVALID) printf("Can't open %s\n",fullname);
		else printf("%s is not a Minipic\n",fullname);
		exit(1);
	}

  	status = mini2bmp(outfile);
	if (status == INVALID) printf("Can't open %s.BMP\n",outfile);
	else printf("%s.BMP created!\n",outfile);

 	puts("Have a Nice Dos!");
	exit(0);

}
