/*  ------------------------------------------------------------------------
System       : Manx Aztec C65 Version 3.2b
               MS-DOS cross-development environment
Platform     : Apple IIe 128K PRODOS 8
Program      : bin2ragC) Copyright Bill Buckels 2013.
               All rights reserved.

Description  : Apple IIe ProDOS 8 HGR Graphics File Utility Program

			   HGR formats supported:

			   BSAVED HGR Images - .BIN Files
			   HGR Raw Raster Images - .TOP, .BOT, and .RAG Files

               Uses Page 2 Hi-Res to avoid
               ProDOS Sys Program Load Address at 0x2000

This Utility serves 2 main purposes:

1. Round-Trip Conversion between BSaved fullscreen HGR Images (.BIN
files) and fullscreen and mixedscreen Raster Oriented HGR Images
(.BOT and .TOP files), and Raster Oriented HGR Image Fragments (.RAG
files).

2. Produces Raster Oriented HGR Image Fragments (.RAG files) from
all the formats noted above.

Please read the documentation and source code (this file) and run
the program for more information.


Written by   : Bill Buckels
Date Written : March 2013

Revision     : 1.0 First Release
Licence      : You may use this program for whatever you wish as long
               as you agree that Bill Buckels has no warranty or
               liability obligations whatsoever from said use.
------------------------------------------------------------------------ */

#include <g3.h>
#include <stdio.h>
#include <fcntl.h>
#include <prodir.h>
#include <console.h>

/* table of raster origins for HGR2 */
extern unsigned HB[];

foldericon()
{
	/* The following displays two MouseText characters
	   which create a folder icon..*/

#asm

             LDA #$1B           ;MOUSETEXT ON
             JSR $FDED          ;SEND IT THROUGH THE COUT ROUTINE

             LDA #$58           ;SHOW X
             JSR $FDED          ;SEND IT THROUGH THE COUT ROUTINE

             LDA #$59           ;SHOW Y
             JSR $FDED          ;SEND IT THROUGH THE COUT ROUTINE

             LDA #$18           ;MOUSETEXT OFF
             JSR $FDED          ;SEND IT THROUGH THE COUT ROUTINE

#endasm

}

/* the files that we process in here */
unsigned char extensions[4][5] = {
	'.','B','I','N',0,
	'.','R','A','G',0,
	'.','B','O','T',0,
	'.','T','O','P',0};

int getimagetype(picname)
char *picname;
{

	int idx=0;
	char *ptr;

	while(picname[idx]!=0)
	  {
		/* check for file extension */
		if(picname[idx] == '.') {
			ptr = (char *) &picname[idx+1];
			if (ptr[3] != 0) break;
			if (ptr[0] == 'R' && ptr[1] == 'A' && ptr[2] == 'G') return 1;
			if (ptr[0] == 'B' && ptr[1] == 'O' && ptr[2] == 'T') return 2;
			if (ptr[0] == 'T' && ptr[1] == 'O' && ptr[2] == 'P') return 3;
			break;
		}
		idx++;
	}

return 0;
}

/* structures for filefind */
/* point to unused memory below SYS programs */
struct filefind *ff = (struct filefind *)4192;
struct fileinfo *fi = (struct fileinfo *)4192;

printdir()
{
	FILE *fp = NULL;
	char *ext = NULL, *f=NULL, picname[17], pathname[64];
	int filetype = 6, cnt=0, filematch, err = -1, row = 2,col = 4;
	/* cursor is set to 0,0 */
	puts("Bin2Rag(C) Copyright Bill Buckels 2013. All Rights Reserved.");


    if (getprefix (pathname) != -1) {
		err = gfindopen(f,fi);
	}

	if (err > -1) {
    	scr_curs(row, 0);
 		foldericon();
 		scr_curs(row, 2);
		puts(pathname);
		row++;
		scr_curs(row, col);
		while (gfindnext(filetype, ext, picname,ff) == 0){
		    if (ff->size > 8192) continue;
            strupr(picname);
			filematch = 0;
			/* assume binary files of 8192 bytes (or almost)
			   are BSaved HGR Images...

			   they can darned-well remove the headers if processing
			   files from DOS 3.3.

			   */
			if (ff->size > 8183 && ff->size < 8193) filematch = 8192;
			else filematch = getimagetype(picname);

			if (filematch != 0) {
				puts(picname); col+= 16; cnt++;
				if (cnt%4 == 0) {
					row++;
					col = 4;
				}
				scr_curs(row, col);
			}

		}
	}
	if (cnt != 0) {
		if (cnt%4 != 0)row++;
		row++;
	}
	scr_curs(row,0);
	puts("Enter Filename (Blank to Exit):");
	scr_curs(row, 32);

}

/* xored box */
xbox(x1,y1,x2,y2)
{
	unsigned char *ptr;
	int x;

    /* top */
    ptr = (unsigned char *)HB[y1];
    x2++;
    for (x = x1; x < x2; x++)ptr[x]^='\x7f';
    if (y1 == y2) return;

    /* bottom */
    ptr = (unsigned char *)HB[y2];
    for (x = x1; x < x2; x++)ptr[x]^='\x7f';
    x2--;
    y1++;

    /* sides */
    while (y1 < y2) {
		ptr = (unsigned char *)HB[y1];
		ptr[x1]^='\x7f';
		ptr[x2]^='\x7f';
		y1++;
	}
}



/*
	CLR80VID = $C00C ;disable 80-column display mode (WR-only)
 	CLR80STORE=$C000 ;80STORE Off- disable 80-column memory mapping (Write)

*/
unsigned char *clr80vid = (unsigned char *)0xc00c;
unsigned char *clr80store = (unsigned char *)0xc000;

setvideomode(vmode)
int vmode;
{
	switch(vmode) {
		case TEXT80:
		    clear_bottom();
			text();
			crt80();
			scr_clear();
		    scr_curs (0, 0);
		    break;

		case FULLMODE:
			scr_clear();
		    scr_curs (0, 0);
			*clr80store = 0;
			black();
			*clr80vid = 0;
			clear_bottom();
			print_bottom("S=Save  SPACE=Menu  DEL=Delete  ESC=Exit",0,0);
			print_bottom("B=BSave RETURN=View F,T,X=Reset Size",1,0);
			print_bottom("U,D,L,R=Move UP,DN,RT,LT  ARROWS=ReSize",2,0);
			print_bottom("O,G=PalColor 0-7=BackColor I=Inverse",3,0);
			break;
		}

}


clearscreen(color)
int color;
{
	switch(color)
	{
	/* 	 black    =0
		 green    =1
		 violet   =2
		 white    =3
		 black    =4
		 red      =5
		 blue     =6
		 white    =7
	*/
	   case 1:  green(); break;
	   case 2:  violet();break;
	   case 5:  red();   break;
	   case 6:  blue();  break;
	   case 3:
	   case 7:  white(); break;
	   default: black();
	}


}


/* sets single palette (green or orange) or
   inverse video */
setmsk(c)
unsigned char c;
{
	unsigned x, y;
	unsigned char *ptr;

	for (y = 0; y< 192; y++) {
	   ptr = (unsigned char *) HB[y];
	   if (c == 'I')for (x = 0; x < 40; x++) *ptr++ ^= '\x7f';
	   if (c == 'G')for (x = 0; x < 40; x++) *ptr++ &= '\x7f';
	   if (c == 'O')for (x = 0; x < 40; x++) *ptr++ |= '\x80';

	}
}


main()
{
	int c,menu,x1,x2,y1,y2,oldx1,oldx2,oldy1,oldy2,savex2,savey2,imagetype;
	char picname[70];

	hgr();
	getch();

	for (;;) {
        setvideomode(TEXT80);

	    /* display HGR files in current directory */
		printdir();
        picname[0] = 0;
		gets(picname); /* get the filename */
		if (picname[0] < 47)break;

        setvideomode(FULLMODE);
        picname[64] = 0;


        /* set default coordinates here.
           if we are loading a rag, ragload will
           set the initial bounds of the clip
           region to the image fragment size */

        /* set clip region origin to 0,0 */
		oldx1 = x1 = oldy1 = y1 = 0;

		/* set clip region bounds to fullscreen
		   if we are loading an image fragment
		   these will be modified */
		x2 = 39;
		y2 = 191;

        imagetype = getimagetype(picname);
        if (imagetype == 0) c = bload(picname);
		else c = ragload(picname,&x2,&y2);

		savex2 = oldx2 = x2; savey2 = oldy2 = y2;

		if (c!=0) {
			setvideomode(TEXT80);
			if (c == -1) puts("Can't open!");
			if (c == -2) puts("Unsupported format!");
			puts("Press any Key...");
			getch();
			continue;
		}

		xbox(x1,y1,x2,y2);
		menu = 0;

		/* menu loop - stays here until user saves or aborts */
		for(;;) {
			c = toupper(getch());

			if (c == ESCAPE || c == DELETE || c == 'S' || c == 'B') break;

			switch(c) {

                /* set common palette in orange-blue or green-violet */
                case 'O':
				case 'G':
				/* inverse video */
				case 'I':
				            xbox(x1,y1,x2,y2);
				            setmsk((unsigned char)c);
				            xbox(x1,y1,x2,y2);
				            break;

				/* clearscreen to the color specified and reload image */
				case '0':
				case '1':
				case '2':
				case '3':
				case '4':
				case '5':
				case '6':
				case '7':   clearscreen(c-48);
				       		/* reload the image now */
				            /* not checking for errors here...
				               likely if image loaded once it will
				               load again unless the disk is no longer
				               in the drive or similar in which case
				               they can press escape and fix the problem */
       						if (imagetype == 0)bload(picname);
							else ragload(picname,&c,&c);
							/* redo the clipbox */
							xbox(x1,y1,x2,y2);
			                break;

				/* adjust clip box from bottom left  */
				case LTARROW: /* horizontal increments of
				                 2 bytes to preserve color map
				                 and to avoid splitting pixels */
				              x2-=2;
							  if (x2 < x1)x2+= 2;
							  break;
				case RTARROW: x2+=2;
							  if (x2 > 39) x2 = 39;
							  break;
				case UPARROW: /* upward increments of 1 raster */
				              y2--;
							  /* allow a single raster rag file */
							  if (y2 < y1) y2 = y1;
							  break;
				case DNARROW: /* downward increments of 4 rasters */
				              y2+=4;
							  if (y2 > 191) y2 = 191;
							  break;
				/* move clip box */
				case 'U': /* move up by one raster */
						  if (y1 < 1) break;
						  y1--; y2--; break;

				case 'D': /* move down by 4 rasters */
						  if (y2 < 191) { y2++; y1++;}
						  if (y2 < 191) { y2++; y1++;}
						  if (y2 < 191) { y2++; y1++;}
						  if (y2 < 191) { y2++; y1++;}
						  break;

                /* note - left and right are reversed
                   in this case intuition and keyboard layout
                   overules logic and convention so I went with
                   the flow here */
				case 'R': /* move left by 2 bytes */
						  if (x1 < 2) break;
						  x1-=2; x2-=2; break;

				case 'L': /* move right by 2 bytes */
						  if (x2 > 37) break;
						  x1+=2; x2+=2; break;

				/* reset for BOT or TOP */
				case 'X': x1=0; x2=savex2; y1=0;y2 = savey2; break;
				case 'F': x1=0; x2=39; y1=0; y2=191; break;
				case 'T': x1=0; x2=39; y1=0; y2=159; break;
				/* get rid of the clipbox and hide the menu if posing
				   for a screenshot... display the menu after they are
				   done so they know we are back-from-the-dead */
                case ENTERKEY: fscreen();
                               xbox(x1,y1,x2,y2);
                               getch();
                               xbox(x1,y1,x2,y2);
                               menu = 0;
				case SPACEBAR:
						  /* menu toggle */
						  if (menu == 1) { menu = 0; fscreen();}
						  else { menu = 1; mscreen();}
						  break;

			}

			/* if coordinates have changed erase old xbox and draw new xbox */
			if (oldx1 != x1 || oldx2 != x2 || oldy1 != y1 || oldy2 != y2) {
				xbox(oldx1,oldy1,oldx2,oldy2);
				xbox(x1,y1,x2,y2);
				oldx1 = x1;
				oldx2 = x2;
				oldy1 = y1;
				oldy2 = y2;
			}

		}

		/* save or delete RAG or Bsaved Image */
		if (c == 'S' || c=='B' || c==DELETE ) {
			xbox(x1,y1,x2,y2);
			fscreen();
			text();
			scr_clear();
			scr_curs (0, 0);
			if (c == DELETE)
			     puts("Delete?? (Y/N)?");
	        else
	             puts("Enter Savename:");
	        scr_curs(0, 15);
	        aputc(32,stdout);
			if (c == DELETE) {
				if (toupper(getch()) == 'Y') unlink(picname);
				continue;
			}
			gets(picname); /* get the filename */
			if (picname[0] > 46) {
				hgr();
				c = ragsave(picname, x1,y1,x2,y2, (unsigned char)c);
			    setvideomode(TEXT80);
			    if (c == 0)puts("Saved!");
			    if (c == -1)puts("Can't open!");
			    if (c == -2)puts("Can't save!");
			    if (c == -3)puts("Can't set filetype!");
			    puts("Press any Key...");
		        getch();
			}
		}
	}

	scr_clear();
	scr_curs (0, 0);
    _exit(0);
}


int bload(name)
char *name;
{
   int fh, c=-1;
   if ((fh = open(name,O_RDONLY,0xc3))!=-1) {
   		c = read(fh,(char *)HB[0],0x2000);
   		close(fh);
   		if (c > 8183 && c < 8193) c = 0;
   		else c = -2;
	}
	return c;
}


/* terse version of piclode - load from top left */
int ragload(name,x2,y2)
char *name;
int *x2,*y2;
{
   int fh,y,height=193,packet=41,c=-2;
   char tempchar[2];

   fh = open(name,O_RDONLY,0xc3); /* open a binary file */
   if (fh == -1)return -1;

   if (read(fh,tempchar,2)==2) {
	   packet= (int)tempchar[0];
	   height= (int)tempchar[1];
   }

   /* use maximum bounds for validation test */
   if (height < 193 && packet < 41) {
	   for(y=0;y<height;y++) {
		 /* read each raster to the screen */
		 c = read(fh,(unsigned char *)HB[y],packet);
		 if(c!=packet) break;
	   }
	   if (c != packet) c = -2;
	   else {
		   /* set bounds of clipping region */
		   x2[0] = packet - 1;
		   y2[0] = height - 1;
		   /* success */
		   c = 0;
	   }
   }
   close(fh);
   return c;
}

ragsave(name, x1, y1, x2, y2, savetype)
char *name,savetype;
int x1,y1,x2,y2;
{
    int fh, c=-1, width, height,i, e=1;
    unsigned char buf[2], *ptr;

	y2++;
	height = (y2 - y1);
	x2++;
	width = (x2 - x1);

    /* get basename for save */
    for (fh = 0;;) {
		if (name[fh] == '.') name[fh] = 0;
		if (name[fh] == 0) break;
		fh++;
	}

	/* RAG or BSaved Image */
	if (savetype == 'S') {
		/* enforce standard naming convention */
    	if (width == 40 && height == 192) e = 2;
		else if (width == 40 && height == 160) e = 3;
	}
	else e = 0;

	/* add standard extension to basename */
	for (i=0;i<5;i++,fh++) name[fh] = extensions[e][i];

	/* overwrite file if it already exists */
	if((fh=open(name, O_WRONLY|O_TRUNC|O_CREAT,0xc3)) == -1) return c;
	close(fh);

	if (getfinfo (name, fi) == -1) {
	   unlink(name);
	   return -3;
	}

	fi->file_type = 6;
	if (savetype == 'S')
		fi->aux_type = 0x4000; /* RAG - screen 2 */
	else
		fi->aux_type = 0x2000; /* BSaved - screen 1 */

	setfinfo(name,fi);

	if((fh=open(name, O_WRONLY,0xc3)) != -1) {

	    /* save as RAG or BSaved Image */
		if (savetype == 'S') {
			/* RAG */
			buf[0] = (unsigned char) width;
			buf[1] = (unsigned char) height;
			c = write(fh,buf,2);
			if (c != 2) c = -2;
			else {
				while (y1 < y2) {
				   ptr = (unsigned char *) (HB[y1] + x1);
				   c = write(fh,&ptr[0],width); /* write each raster */
				   if (c!= width) break;
				   y1++;
				}
				if (c != width) c = -2;
				else c = 0;
			}
		}
		else {
			/* BSaved Image */
			c = write(fh,(char *)HB[0],0x2000);
   			if (c == 0x2000) c = 0;
   			else c = -2;
		}
        close(fh);
    }

    if (c!= 0)unlink(name);

    return c;
}

/* some functions to output text directly to the screen */

/* base addresses for last 4 text lines */
/* of secondary text page */
int textbase[4]={
    0x0A50,
    0x0AD0,
    0x0B50,
    0x0BD0};

/* clear the last 4 lines */
clear_bottom()
{
	char *crt;
	int row, col;
	char c = 32 + 128;

	for (row = 0; row < 4; row++) {
	  crt = (char *)(textbase[row]);
      for (col = 0; col < 40; col++) {
	     *crt++ = c;
	  }
    }
}

/* row = 0,0 to 3,39 in mixed mode */
print_bottom(str,row,col)
char *str;
int row, col;
{
     char *crt  = (char *)(textbase[row]+col);
     char c;

      while((c=*str++)!=0)*crt++=(c+128);
}


/* utility function - set string to upper case
   useful for string comparisons and squinty displays
   from back in the day when uppercase was not considered
   shouting */
strupr(str)
char *str;
{
	int i;
	char az;

	for (i = 0; str[i] != 0; i++) {
		az = str[i];
		if (az > 96 && az < 123) str[i] = (az - 32);
	}
}