/* ------------------------------------------------------------------------
System       : Manx Aztec C65 Version 3.2b
               MS-DOS cross-development environment
Platform     : Apple IIe 128K PRODOS 8
Program      : shlode(C) Copyright Bill Buckels 2013.
               All rights reserved.
Description  : Apple //e ProDOS 8 HGR Command Line Picture Viewer
               for the Aztec C ProDOS Shell.

               shlode commandline usage: shlode -seconds HGRimagename.BIN
			                             shlode -seconds HGRimagename.TOP
			                             shlode -seconds HGRimagename.BOT
			                             shlode -seconds HGRimagename.RAG
			                             shlode -seconds HGRimagename.RAX
			   shlode interactive usage: shlode

               HGR formats supported:

                       BSAVED HGR Images - .BIN Files
                       HGR Raw Raster Images - .TOP, .BOT, and .RAG Files
                       HGR Run-Length Encoded Images - .RAX Files

Written by   : Bill Buckels
Date Written : February 2013
Revision     : 1.0 First Release
Licence      : You may use this program for whatever you wish as long
               as you agree that Bill Buckels has no warranty or
               liability obligations whatsoever from said use.
------------------------------------------------------------------------ */
#include <g3.h>
#include <stdio.h>
#include <console.h>

extern unsigned char XPIC;
extern unsigned char YPIC;

unsigned char raxreadbuf[1024];

/*
	CLR80VID = $C00C ;disable 80-column display mode (WR-only)
 	SET80VID = $C00D ;enable 80-column display mode (WR-only)

unsigned char *clr80vid = (unsigned char *)0xc00c;
unsigned char *set80vid = (unsigned char *)0xc00d;

	SET80STORE=$C001 ;80STORE On- enable 80-column memory mapping (WR-only)
	CLR80STORE=$C000 ;80STORE Off- disable 80-column memory mapping (Write)

unsigned char *set80store = (unsigned char *)0xc001;

*/

unsigned char *clr80store = (unsigned char *)0xc000;

/* terse version of getimagetype for this program
   based purely on my naming convention */
int getimagetype(picname)
char *picname;
{

	int idx=0,jdx=4;
	char *ptr;

    /* null pointer */
    if (NULL == picname) return -1;
    /* empty string */
    if (picname[0] == 0) return -1;

	while(picname[idx]!=0)
	  {
		/* garbage characters */
		if(picname[idx]< 46 || picname[idx] > 122) return -1;

		/* check for file extension */
		if(picname[idx] == '.' &&  picname [jdx] == 0) {
			ptr = (char *) &picname[idx+1];

			if (ptr[0] == 'B' && ptr[1] == 'O' && ptr[2] == 'T') return 1;
			if (ptr[0] == 'T' && ptr[1] == 'O' && ptr[2] == 'P') return 2;
			if (ptr[0] == 'R' && ptr[1] == 'A' && ptr[2] == 'G') return 3;
			if (ptr[0] == 'R' && ptr[1] == 'A' && ptr[2] == 'X') return 4;

            /* assume bsaved image */
			return 0;

		}
		idx++;jdx++;
	}

/* assume bsaved image */
return 0;
}

char *usage =
"shlode(C) Copyright Bill Buckels 2013.All rights reserved.\n"
"Command Line: shlode HGRimagename or shlode -seconds HGRimagename\n"
"  Multiple images and wildcards can be entered for slideshow\n"
"  Slideshow Timeout is in seconds (default = 5)\n"
"HGR formats supported:\n"
"BSAVED HGR Images - .BIN Files\n"
"HGR Raw Raster Images - .TOP, .BOT, and .RAG Files\n"
"HGR Run-Length Encoded Images - .RAX Files";


/* 20 pictures */
/* a fully prefixed image path can be 65 characters with the null terminator */
/* my buffer size for storing the image names from the commandline is
   20 x 65 = 1300 bytes */

#define MAXPIX 20
#define PIXLEN 65

char picnamebuf[MAXPIX * PIXLEN];
char *picnames[MAXPIX];


/* slideshow timeout 1-100 if more than one image loaded from the
commandline - default is 5 seconds (approx.) on a "stock" apple II
and if the user has an accelerator or wishes something different
this can be adjusted as necessary. */

main (argc, argv)
int argc;
char *argv[];
{

	char *picname, *ptr;
	int c= -3, cnt, idx, numpics = 0, timeout = 5, key, imagetype;

    /* initialize array */
    for (cnt = 0, idx=0; cnt < MAXPIX; cnt++, idx+= PIXLEN) {
	   picnames[cnt] = (char *) &picnamebuf[idx];
	   picnamebuf[idx] = 0;
	}

    if (argc > 1) {
		/* getargs - time-out or image name ? */
		for (cnt = 1; cnt < argc; cnt++) {
			/* switch character is NOT optional */
			if (argv[cnt][0] == '-') {
				ptr = (char *)&argv[cnt][1];
				idx = atoi(ptr);
				if (idx > 100) idx = 100;
				if (idx > 0) timeout = idx;
			}
			else {
				/* get each picture name from the command line */
				ptr = (char *)&argv[cnt][0];
				/* validate picture name */
				strupr(ptr);
				if (getimagetype(ptr) == -1) continue;
				picname = picnames[numpics];
				strcpy(picname,ptr);
				numpics++;
				if(numpics >= MAXPIX)break; /* maximun of 20 pictures */
			}
		 }
 	 }
 	 else {
		/* if interactive mode display title for 10 seconds
		   or until a key is pressed */

		while (kbhit())clearkey(); /* clear keyboard */
		*clr80store = 0;
		hgr();
		wait(10,XTIME,YTIME);
		while (kbhit())clearkey();
		text();
		crt80();
		scr_clear();
		scr_curs (0, 0);
		puts(usage);
		scr_curs(9,0);
		puts("Enter HGR Pic Name: ");
		scr_curs(9,20);
		picname = picnames[0]; /* point to first picture */
		gets(picname); /* get the filename */
		scr_clear();
		scr_curs (0, 0);
		if (picname[0] == 0)exit(c); /* blank line to end program */
		strupr(picname);
		if (getimagetype(picname) != -1) numpics = 1;
	}

	*clr80store = 0;
	black();

	cnt = 0;
    while (kbhit())clearkey();

	while (numpics != 0) {


        /* preload keycheck */
        key = kbhit();
		if (key!=0) {
			clearkey();
			if (key == ESCAPE) break; /* if escape key is pressed end slideshow */
			key = 0;
		}

		picname = picnames[cnt];


		/* load picture */
		/* get the image type and parse the load coordinates if any */
		imagetype = getimagetype(picname);


		switch(imagetype)
		{            /* centre compressed images on the screen */
			case 4:  c = raxlode(picname,40,192,1024,( char *)&raxreadbuf[0]);
					 break;
			case 3:  /* centre image fragments on the screen */
					 XPIC=40;
					 YPIC=192;
			case 2:
			case 1:  /* do not centre bot and top images */
					 c = piclode(picname);
					 break;
			default: c = bload(picname);
		}

		/* if a single picture we are not running a slideshow *
			1. if a problem just exit,
			2. otherwise wait for key press and exit */
		if (numpics < 2) {
			if (c<0) break;
			getch();
			break;
		}

		/* advance on timeout or advance, reverse or end on keypress */
		if (c==0) key = wait(timeout,XTIME,YTIME);
		else {
			key = kbhit();
			if (key!=0)clearkey();

		}

		/* postload key check */
		if (key == ESCAPE) break; /* if escape key is pressed end slideshow */

	   	switch(key) {
         	case UPARROW : /* reverse direction */
            case LTARROW : cnt--;
                           if (cnt < 0) cnt = (numpics - 1);
                           break;
			default: cnt++;
		}

        /* get next slide */
        if (cnt >= numpics) cnt = 0;
	}

	while (kbhit())clearkey();

    text();
	crt80();
    scr_clear();
    scr_curs (0, 0);

    /* if a picture did not load and we are not running a slideshow
       report an error */

	if (c<0 && numpics<2) {
	  if (c == -1) puts("!Can't open pic!");
	  else puts("!Pic is in an unsupported format!");
	  puts("Press Any Key...");
	  getch();
 	  scr_clear();
      scr_curs (0, 0);

	 }

	exit(c);

}
