
/* ------------------------------------------------------------------------
System       : Manx Aztec C65 Version 3.2b
               MS-DOS cross-development environment
Platform     : Apple IIe 128K PRODOS 8
Program      : hcubes.c
Description  : G2 Library Demo Program
               Isometric Cube Drawing Program in
               Hi-Res 280 x 192 monochrome mode.
Written by   : Bill Buckels
Date Written : February 2013
Revision     : 1.0 First Release
Licence      : You may use this routine for whatever you wish as long
               as you agree that Bill Buckels has no warranty or
               liability obligations whatsoever from said use.
------------------------------------------------------------------------ */


/* This program calculates the dimensions required to produce a
"Worms's Eye View" of an isometric cube from height, length, and
angle and then draws the cube.

It is based on a test program called CUBES.BAS that I wrote in BASIC
for the IBM-PC in October, 1989 when I was first developing
commercial 3-D simulations for "truecolor" Kiosk Systems for the Home
Decorating Industry. Today it makes a reasonable demo.

In passing, I should also note that CUBES.BAS used cartesian
coordinates rather than screen coordinates which resulted in a bird's
eye view rather than a worm's eye view of the isometric cube.

Also, for this demo I have renamed the variables from CUBES.BAS where
they might be confusing and adjusted the limits appropriately for the
apple II display using a 480 x 480 viewport for my calculations and
then I scale the yterm by 1:3 for plotting and I scale the xterm by
1:2 which results in a 240 x 160 display resolution for the cube.
CUBES.BAS was targeted at a 512 x 512 display and used a scaled
viewport which did not require scaling adjustments for the display
since scaling was done by the BASIC graphics library.

CUBES.BAS also filled the facets of the cube with fill patterns.
Due to laziness on my part I have never written a fill function
for any of the Apple II screen modes in my Aztec C graphics library
so for this demo I have simply ported the line drawing calculations
and line plotting and make no attempt to fill the cube.

*/

double atof();
double height, length, angle;

cube(color)
int color;
{

  double baselength, elev, xorg, yorg, midbot, midtop, botside, boxbot, ltedge, rtedge, htedge;
  int x, x1, x2, y, y1, y2, y3;

  /* reduce the length by the rate of deflection in degrees from base 0 */
  baselength=(length-((length/90)*angle));

  /* subtract the rate of deflection from the height */
  elev=((height/90)*angle);

  /* offset the cube from the middle of the screen */
  xorg=(240.0);             /* horizontal viewpoint */
  yorg=(240.0-elev);        /* vertical viewpoint */
  midtop= (yorg-(height/2));/* top apex */
  botside=(height+midtop);  /* leading edge */
  midbot= (botside+elev);   /* bottom dead centre */
  boxbot= (midbot+elev);    /* bottom apex */
  ltedge= (xorg-baselength);/* left hand length truncation */
  rtedge= (xorg+baselength);/* right hand length truncation */
  htedge= (elev+midtop);    /* elevation at edges */

  /* draw the cube left to right, bottom to top */

  /* centre xterm on view and scale down xterm from 240 to 140 */
  x = (int)(double)((40.0 + xorg)/2.0);
  x1 = (int)(double)((40.0 + rtedge)/2.0);
  x2 = (int)(double)((40.0 + ltedge)/2.0);

  /* scale down yterm from 480 to 160 */
  y = (int) (midbot /3);
  y1 = (int) (boxbot / 3);
  y2 = (int) (botside / 3);

  /* draw the bottom */
  monoline(x2,y,x,y1,color);
  monoline(x,y1,x1,y,color);
  monoline(x1,y,x,y2,color);
  monoline(x,y2,x2,y,color);

  /* scale down yterm from 480 to 160 */
  y1 = (int) (midtop / 3);
  y3 = (int) (htedge /3);

  /* draw the left side */
  monoline(x2,y,x2,y3,color);
  monoline(x2,y3,x,y1,color);
  monoline(x,y1,x,y2,color);
  monoline(x,y2,x2,y,color);

  /* draw the right side */
  monoline(x1,y,x1,y3,color);
  monoline(x1,y3,x,y1,color);
  monoline(x,y1,x,y2,color);
  monoline(x,y2,x1,y,color);

}

main()
{

 char buf[40];
 int drawcolor=1, h, l, a;

 /* set hires */
 /* embedded splash screen */
 hgr();
 getch();

 for (;;) {
	text();
	scr_clear();
 	scr_curs (0, 0);

    /* viewport set to 480 x 480 */
    printf("HCubes - A Demo by Bill Buckels\n");
    printf("Enter blanks for default...\n");
 	printf("Box Height (10 to 240): "); gets(buf); height=atof(buf);
 	printf("Box Length (10 to 240): "); gets(buf); length=atof(buf);
 	printf("Box Angle  (10 to 80):  ");  gets(buf); angle=atof(buf);

    a = (int)angle;
    l = (int)length;
    h = (int)height;

	if (a < 10 || a > 80) angle = 30.0;
    if (l < 10 || l > 240) length = 240.0;
	if (h < 10 || h > 240) height = 240.0;

    a = (int)angle;
    l = (int)length;
    h = (int)height;

    sprintf(buf,"H=%d L=%d A=%d Wait!",h,l,a);
    if (drawcolor == 0) white();
	else black();
    monoplots("HCubes - A Demo by Bill Buckels",139,0,drawcolor,'M');
    clear_bottom();
    mscreen();
    print_bottom(buf,2,0);

    cube(drawcolor);
    print_bottom("Esc exits! Other key continues!",2,0);

  	if (getch()==27)break;

 	if (drawcolor == 1) drawcolor = 0;
	else drawcolor = 1;

	}

 black();
 text();
 scr_clear();
 scr_curs (0, 0);

#ifdef SHPRG
/* return to the shell */
 exit(0);
#else
/* return to ProDOS */
 _exit();
#endif

}