/* ------------------------------------------------------------------------
System       : Manx Aztec C65 Version 3.2b
               MS-DOS cross-development environment
Platform     : Apple IIe 128K PRODOS 8
Program      : dl.c
Description  : G2 Library Demo and Test Program
               Directory List

               This program lists the files (names only) in the current directory.

			   The Aztec C Shell version also provides a commandline
			   pathname option to list the files in directories other
			   than the current directory and to list files by type and or
			   file extension. partial extension wildcards pattern character '?'
			   can be used when specifying extensions.

			   The shell version can also be redirected to a disk file.

               2 versions of this program are built with the
			   MAKEFILE:

               1. A ProDOS SYS Program
               2. An Aztec C Shell PRG Program

               Separate Diskimages for each are also provided.

               Note the defines in the code below that are required
			   for compiler options to build separate programs for
			   these 2 different environments.

               I used this program as a prototype to develop filefind
			   routines for the G2 library, and left it here
			   afterwards because I think it should prove
			   informative.

Written by   : Bill Buckels
Date Written : January 2013
Revision     : 1.0 First Release
Licence      : You may use this routine for whatever you wish as long
               as you agree that Bill Buckels has no warranty or
               liability obligations whatsoever from said use.
------------------------------------------------------------------------ */

#include <stdio.h>
#include <prodir.h>

struct filefind ff;

#ifdef SHELLPRG

main (argc, argv)
int argc;
char *argv[];
{

#else

main ()
{

#endif

	char pathname[64], filename[16], extension[16], *ext = NULL, *f=NULL, c, *ptr;
	int filetype=0, err = 0, cnt;

#ifdef SHELLPRG
    if (argc > 1){
		pathname[0] = 0;

		for (cnt = 1; cnt < argc; cnt++) {
		/* switch character is not optional */
			if (argv[cnt][0] == '-') {
				c = argv[cnt][1];
				if (c == 't' || c == 'e') {
					ptr = (char *)&argv[cnt][2];
					while (*ptr == ' ')*ptr++;

					if (c == 't') {
						filetype = atoi(ptr);
					}
					else {
						strcpy(extension, ptr);
						ext = (char *)&extension[0];
					}
				}
			}
			else {
			   strcpy(pathname,argv[cnt]);
			   f = (char *)&pathname[0];

			}
		}
	}

	err = findopen(f);

	if (err > -1) {
		while (findnext(filetype, ext, filename, &ff) == 0)printf("%s\n",filename);
		err = 0;

	}
	else printf("err=%d\n",err);


	exit(err);

#else

    char oldpath[64];

    crt80(); /* set to 80 column mode */
	scr_clear();
	scr_curs(0,0);

	/* no commandline in ProDOS so get interactive input */
	printf("dl (directory list) demo program by Bill Buckels 2013\n");
	printf("Enter file search spec below... (blank for default - all)\n");
	printf("Pathname (/Prefix/)      :"); gets(pathname);
	printf("ProDOS Filetype (Decimal):"); gets(filename); /* just using filename as a buffer */
	printf("Extension (?=wildcards)  :"); gets(extension);

	if(pathname[0] != 0) f = (char *)&pathname[0];
	if(filename[0] != 0) filetype = atoi(filename);
	if(extension[0] != 0) ext = (char *)&extension[0];

    getprefix (oldpath);

	scr_clear();
	scr_curs(0,0);

	err = findopen(f);

	if (err > -1) {
		cnt = 0;
		while (findnext(filetype, ext, filename, &ff) == 0){
			printf("%s\n",filename);
			cnt++;
            if (cnt > 20) {
			  reversetext();
			  printf("\nPress Any Key...");
			  getch();
              normaltext();
			  scr_cr();  		/* put cursor in column 0 of next line */
			  scr_cursup();		/* goto previous line */
			  scr_ldelete();	/* delete previous line */
			  scr_cursup();		/* goto previous line */
			  scr_ldelete();	/* delete previous line */
			  cnt=0;
			}
		}
		setprefix (oldpath);
	}
	else printf("err=%d\n",err);

    getch();
	scr_clear();
	scr_curs(0,0);
	_exit();

#endif

}